@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,6 +20,7 @@
 """InfluxDB (up to InfluxDB 1.7.x) interface class."""
 
 import sys
+from platform import node
 
 from glances.logger import logger
 from glances.exports.glances_export import GlancesExport
@@ -56,6 +57,9 @@ class Export(GlancesExport):
         if not self.export_enable:
             sys.exit(2)
 
+        # The hostname is always add as a tag
+        self.hostname = node().split('.')[0]
+
         # Init the InfluxDB client
         self.client = self.init()
 
@@ -93,32 +97,57 @@ class Export(GlancesExport):
         return db
 
     def _normalize(self, name, columns, points):
-        """Normalize data for the InfluxDB's data model."""
+        """Normalize data for the InfluxDB's data model.
+        Output is a list of measurements."""
+        ret = []
 
-        for i, _ in enumerate(points):
-            # Supported type:
-            # https://docs.influxdata.com/influxdb/v1.5/write_protocols/line_protocol_reference/
-            if points[i] is None:
-                # Ignore points with None value
-                del(points[i])
-                del(columns[i])
-                continue
-            try:
-                points[i] = float(points[i])
-            except (TypeError, ValueError):
-                pass
-            else:
-                continue
-            try:
-                points[i] = str(points[i])
-            except (TypeError, ValueError):
-                pass
-            else:
-                continue
+        # Build initial dict by crossing columns and point
+        data_dict = dict(zip(columns, points))
 
-        return [{'measurement': name,
-                 'tags': self.parse_tags(self.tags),
-                 'fields': dict(zip(columns, points))}]
+        # issue1871 - Check if a key exist. If a key exist, the value of
+        # the key should be used as a tag to identify the measurement.
+        keys_list = [k.split('.')[0] for k in columns if k.endswith('.key')]
+        if len(keys_list) == 0:
+            keys_list = [None]
+
+        for measurement in keys_list:
+            # Manage field
+            if measurement is not None:
+                fields = {k.replace('{}.'.format(measurement), ''): data_dict[k]
+                          for k in data_dict
+                          if k.startswith('{}.'.format(measurement))}
+            else:
+                fields = data_dict
+            # Transform to InfluxDB datamodel
+            # https://docs.influxdata.com/influxdb/v1.8/write_protocols/line_protocol_reference/
+            for k in fields:
+                #  Do not export empty (None) value
+                if fields[k] is None:
+                    fields.pop(k)
+                # Convert numerical to float
+                try:
+                    fields[k] = float(fields[k])
+                except (TypeError, ValueError):
+                    # Convert others to string
+                    try:
+                        fields[k] = str(fields[k])
+                    except (TypeError, ValueError):
+                        pass
+            # Manage tags
+            tags = self.parse_tags(self.tags)
+            if 'key' in fields and fields['key'] in fields:
+                # Create a tag from the key
+                # Tag should be an string (see InfluxDB data model)
+                tags[fields['key']] = str(fields[fields['key']])
+                # Remove it from the field list (can not be a field and a tag)
+                fields.pop(fields['key'])
+            # Add the hostname as a tag
+            tags['hostname'] = self.hostname
+            # Add the measurement to the list
+            ret.append({'measurement': name,
+                        'tags': tags,
+                        'fields': fields})
+        return ret
 
     def export(self, name, columns, points):
         """Write the points to the InfluxDB server."""
@@ -130,7 +159,8 @@ class Export(GlancesExport):
             logger.debug("Cannot export empty {} stats to InfluxDB".format(name))
         else:
             try:
-                self.client.write_points(self._normalize(name, columns, points), time_precision="s")
+                self.client.write_points(self._normalize(name, columns, points),
+                                         time_precision="s")
             except Exception as e:
                 # Log level set to debug instead of error (see: issue #1561)
                 logger.debug("Cannot export {} stats to InfluxDB ({})".format(name, e))
