@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -24,27 +24,17 @@ import sys
 import tempfile
 
 from glances import __version__, psutil_version
-from glances.compat import input
+from glances.compat import input, disable, enable
 from glances.config import Config
 from glances.globals import WINDOWS
 from glances.logger import logger, LOG_FILENAME
 
 
-def disable(class_name, var):
-    """Set disable_<var> to True in the class class_name."""
-    setattr(class_name, 'disable_' + var, True)
-
-
-def enable(class_name, var):
-    """Set disable_<var> to False in the class class_name."""
-    setattr(class_name, 'disable_' + var, False)
-
-
 class GlancesMain(object):
     """Main class to manage Glances instance."""
 
-    # Default stats' refresh time is 3 seconds
-    refresh_time = 3
+    # Default stats' minimum refresh time is 2 seconds
+    DEFAULT_REFRESH_TIME = 2
     # Set the default cache lifetime to 1 second (only for server)
     cached_time = 1
     # By default, Glances is ran in standalone mode (no client/server)
@@ -77,7 +67,7 @@ Examples of use:
   Monitor local machine and export stats to a CSV file (standalone mode):
     $ glances --export csv --export-csv-file /tmp/glances.csv
 
-  Monitor local machine and export stats to a InfluxDB server with 5s refresh time (standalone mode):
+  Monitor local machine and export stats to a InfluxDB server with 5s refresh rate (standalone mode):
     $ glances -t 5 --export influxdb
 
   Start a Glances XML-RPC server (server mode):
@@ -215,12 +205,14 @@ Examples of use:
                             help='SNMP authentication key (only for SNMPv3)')
         parser.add_argument('--snmp-force', action='store_true', default=False,
                             dest='snmp_force', help='force SNMP mode')
-        parser.add_argument('-t', '--time', default=self.refresh_time, type=float,
-                            dest='time', help='set refresh time in seconds [default: {} sec]'.format(self.refresh_time))
+        parser.add_argument('-t', '--time', default=self.DEFAULT_REFRESH_TIME, type=float,
+                            dest='time', help='set minumum refresh rate in seconds [default: {} sec]'.format(
+                                self.DEFAULT_REFRESH_TIME))
         parser.add_argument('-w', '--webserver', action='store_true', default=False,
                             dest='webserver', help='run Glances in web server mode (bottle needed)')
         parser.add_argument('--cached-time', default=self.cached_time, type=int,
-                            dest='cached_time', help='set the server cache time [default: {} sec]'.format(self.cached_time))
+                            dest='cached_time', help='set the server cache time [default: {} sec]'.format(
+                                self.cached_time))
         parser.add_argument('--open-web-browser', action='store_true', default=False,
                             dest='open_web_browser', help='try to open the Web UI in the default Web browser')
         # Display options
@@ -268,6 +260,8 @@ Examples of use:
         args = self.init_args().parse_args()
 
         # Load the configuration file, if it exists
+        # This function should be called after the parse_args
+        # because the configration file path can be defined
         self.config = Config(args.conf_file)
 
         # Debug mode
@@ -278,6 +272,15 @@ Examples of use:
             from warnings import simplefilter
             simplefilter("ignore")
 
+        # Plugins refresh rate
+        if self.config.has_section('global'):
+            global_refresh = self.config.get_float_value('global',
+                                                         'refresh',
+                                                         default=self.DEFAULT_REFRESH_TIME)
+        if args.time == self.DEFAULT_REFRESH_TIME:
+            args.time = global_refresh
+        logger.debug('Global refresh rate is set to {} seconds'.format(args.time))
+
         # Plugins disable/enable
         # Allow users to disable plugins from the glances.conf (issue #1378)
         for s in self.config.sections():
