@@ -2,7 +2,7 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
 #
 # Glances is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -17,11 +17,10 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-import operator
 import os
 
-from glances.compat import iteritems, itervalues, listitems, iterkeys
-from glances.globals import BSD, LINUX, MACOS, SUNOS, WINDOWS, WSL
+from glances.compat import iterkeys
+from glances.globals import BSD, LINUX, MACOS, WINDOWS
 from glances.timer import Timer, getTimeSinceLastUpdate
 from glances.filter import GlancesFilter
 from glances.logger import logger
@@ -30,7 +29,6 @@ import psutil
 
 
 class GlancesProcesses(object):
-
     """Get processed stats using the psutil library."""
 
     def __init__(self, cache_timeout=60):
@@ -42,7 +40,8 @@ class GlancesProcesses(object):
 
         # The internals caches will be cleaned each 'cache_timeout' seconds
         self.cache_timeout = cache_timeout
-        self.cache_timer = Timer(self.cache_timeout)
+        # First iteration, no cache
+        self.cache_timer = Timer(0)
 
         # Init the io dict
         # key = pid
@@ -50,11 +49,17 @@ class GlancesProcesses(object):
         self.io_old = {}
 
         # Init stats
-        self.auto_sort = True
-        self._sort_key = 'cpu_percent'
+        self.auto_sort = None
+        self._sort_key = None
+        # Default processes sort key is 'auto'
+        # Can be overwrite from the configuration file (issue#1536) => See glances_processlist.py init
+        self.set_sort_key('auto', auto=True)
         self.processlist = []
         self.reset_processcount()
 
+        # Cache is a dict with key=pid and value = dict of cached value
+        self.processlist_cache = {}
+
         # Tag to enable/disable the processes stats (to reduce the Glances CPU consumption)
         # Default is to enable the processes stats
         self.disable_tag = False
@@ -62,6 +67,28 @@ class GlancesProcesses(object):
         # Extended stats for top process is enable by default
         self.disable_extended_tag = False
 
+        # Test if the system can grab io_counters
+        try:
+            p = psutil.Process()
+            p.io_counters()
+        except Exception as e:
+            logger.warning('PsUtil can not grab processes io_counters ({})'.format(e))
+            self.disable_io_counters = True
+        else:
+            logger.debug('PsUtil can grab processes io_counters')
+            self.disable_io_counters = False
+
+        # Test if the system can grab gids
+        try:
+            p = psutil.Process()
+            p.gids()
+        except Exception as e:
+            logger.warning('PsUtil can not grab processes gids ({})'.format(e))
+            self.disable_gids = True
+        else:
+            logger.debug('PsUtil can grab processes gids')
+            self.disable_gids = False
+
         # Maximum number of processes showed in the UI (None if no limit)
         self._max_processes = None
 
@@ -152,6 +179,11 @@ class GlancesProcesses(object):
         else:
             return None
 
+    @property
+    def processes_count(self):
+        """Get the current number of processes showed in the UI."""
+        return min(self._max_processes - 2, glances_processes.processcount['total'] - 1)
+
     @property
     def max_processes(self):
         """Get the maximum number of processes showed in the UI."""
@@ -232,27 +264,40 @@ class GlancesProcesses(object):
 
         # Grab standard stats
         #####################
-        standard_attrs = ['cmdline', 'cpu_percent', 'cpu_times', 'memory_info',
-                          'memory_percent', 'name', 'nice', 'pid', 'ppid',
-                          'status', 'username', 'status', 'num_threads']
-        # io_counters availability: Linux, BSD, Windows, AIX
-        if not MACOS and not SUNOS and not WSL:
-            standard_attrs += ['io_counters']
-        # gids availability: Unix
-        if not WINDOWS:
-            standard_attrs += ['gids']
+        sorted_attrs = ['cpu_percent', 'cpu_times', 
+                          'memory_percent', 'name', 
+                          'status', 'status', 'num_threads']
+        displayed_attr = ['memory_info', 'nice', 'pid', 'ppid']
+        cached_attrs = ['cmdline', 'username']
 
-        # and build the processes stats list (psutil>=5.3.0)
-        self.processlist = [p.info for p in psutil.process_iter(attrs=standard_attrs,
-                                                                ad_value=None)
+        # Some stats are optionals
+        if not self.disable_io_counters:
+            sorted_attrs.append('io_counters')
+        if not self.disable_gids:
+            displayed_attr.append('gids')
+        # Some stats are not sort key
+        # An optimisation can be done be only grabed displayed_attr
+        # for displayed processes (but only in standalone mode...)
+        sorted_attrs.extend(displayed_attr)
+        # Some stats are cached (not necessary to be refreshed every time)
+        if self.cache_timer.finished():
+            sorted_attrs += cached_attrs
+            self.cache_timer.set(self.cache_timeout)
+            self.cache_timer.reset()
+            is_cached = False
+        else:
+            is_cached = True
+
+        # Build the processes stats list (it is why we need psutil>=5.3.0)
+        self.processlist = [p.info
+                            for p in psutil.process_iter(attrs=sorted_attrs,
+                                                         ad_value=None)
                             # OS-related processes filter
                             if not (BSD and p.info['name'] == 'idle') and
                             not (WINDOWS and p.info['name'] == 'System Idle Process') and
                             not (MACOS and p.info['name'] == 'kernel_task') and
                             # Kernel threads filter
-                            not (self.no_kernel_threads and LINUX and p.info['gids'].real == 0) and
-                            # User filter
-                            not (self._filter.is_filtered(p.info))]
+                            not (self.no_kernel_threads and LINUX and p.info['gids'].real == 0)]
 
         # Sort the processes list by the current sort_key
         self.processlist = sort_stats(self.processlist,
@@ -293,7 +338,8 @@ class GlancesProcesses(object):
                     if LINUX:
                         try:
                             extended['memory_swap'] = sum([v.swap for v in top_process.memory_maps()])
-                        except psutil.NoSuchProcess:
+                        except (psutil.NoSuchProcess, KeyError):
+                            # KeyError catch for issue #1551)
                             pass
                         except (psutil.AccessDenied, NotImplementedError):
                             # NotImplementedError: /proc/${PID}/smaps file doesn't exist
@@ -347,6 +393,27 @@ class GlancesProcesses(object):
             # Append the IO tag (for display)
             proc['io_counters'] += [io_tag]
 
+            # Manage cached information
+            if is_cached:
+                # Grab cached values (in case of a new incoming process)
+                if proc['pid'] not in self.processlist_cache:
+                    try:
+                        self.processlist_cache[proc['pid']]= psutil.Process(pid=proc['pid']).as_dict(attrs=cached_attrs,
+                                                                                                    ad_value=None)
+                    except psutil.NoSuchProcess:
+                        pass
+                # Add cached value to current stat
+                try:
+                    proc.update(self.processlist_cache[proc['pid']])
+                except KeyError:
+                    pass
+            else:
+                # Save values to cache
+                self.processlist_cache[proc['pid']] = { cached: proc[cached] for cached in cached_attrs }
+
+        # Apply filter
+        self.processlist = [p for p in self.processlist if not (self._filter.is_filtered(p))]
+
         # Compute the maximum value for keys in self._max_values_list: CPU, MEM
         # Usefull to highlight the processes with maximum values
         for k in self._max_values_list:
@@ -367,10 +434,22 @@ class GlancesProcesses(object):
         """Get the current sort key."""
         return self._sort_key
 
-    @sort_key.setter
-    def sort_key(self, key):
+    def set_sort_key(self, key, auto=True):
         """Set the current sort key."""
-        self._sort_key = key
+        if key == 'auto':
+            self.auto_sort = True
+            self._sort_key = 'cpu_percent'
+        else:
+            self.auto_sort = auto
+            self._sort_key = key
+
+    def kill(self, pid, timeout=3):
+        """Kill process with pid"""
+        assert pid != os.getpid(), "Glances can kill itself..."
+        p = psutil.Process(pid)
+        logger.debug('Send kill signal to process: {}'.format(p))
+        p.kill()
+        return p.wait(timeout)
 
 
 def weighted(value):
